/*                  Part of x2grbl
 *
 *  Copyright Johann Wilhelm <johann.wilhelm@9mal6.de> 2015
 *
 *  see Readme.md for detailed license and usage information!
*/

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QCloseEvent>

#include "graphicspictureitem.h"

#include "gcodeparser.h"
#include "gcodeinterpreter.h"
#include "gcodeoutputdevice.h"
#include "outputdevicegrbl.h"
#include "transferqueue.h"
#include "autolevelproxy.h"

namespace Ui {
class MainWindow;
}

class MainWindow : public QMainWindow
{
    Q_OBJECT
    
public:
    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();
    
    void closeEvent(QCloseEvent *event);
    void writeSettings();
    void readSettings();

private slots:
    void on_selectInputFileButton_clicked();

    void on_actionFit_Preview_triggered();

    void on_actionScale_triggered();

    void on_actionScale_2_triggered();

    void on_runButton_clicked();

    void GrblConnecting();
    void GrblConnected();
    void GrblDisconnected();
    void GrblProbingFinished();

    void GrblFailed();
    void GrblFinished();

    void GrblProgressUpdate(float Progress);

    void on_actionOptions_triggered();

    void on_actionConnect_triggered(bool checked);

    void on_actionProbing_triggered();

    void on_actionMilling_triggered();

    void on_selectHPGLInputFileButton_clicked();

    void on_selectEagleBrdFileButton_clicked();

private:
    Ui::MainWindow *ui;

    double Scale;
    GCodeParser Parser;
    GCodeInterpreter Interpreter;

    AutoLevelProxy AutoLeveler;

    GraphicsPictureItem PictureItem;

    OutputDeviceGrbl Grbl;

};

#endif // MAINWINDOW_H
